// Generated by CoffeeScript 1.7.1
var Stringifier, stream, util;

stream = require('stream');

util = require('util');


/*

`Stringifier([options])`
-----------------------

Options may include:

*   `delimiter`     Set the field delimiter, one character only, defaults to `options.from.delimiter` which is a comma.
*   `rowDelimiter`  String used to delimit record rows or a special value; special values are 'auto', 'unix', 'mac', 'windows', 'unicode'; defaults to 'auto' (discovered in source or 'unix' if no source is specified).
*   `quote`         Defaults to the quote read option.
*   `quoted`        Boolean, default to false, quote all the fields even if not required.
*   `escape`        Defaults to the escape read option.
*   `columns`       List of fields, applied when `transform` returns an object, order matters, read the transformer documentation for additionnal information.
*   `header`        Display the column names on the first line if the columns option is provided.
                    OR create objects with properties named by header titles (when using to.array)
*   `lineBreaks`    String used to delimit record rows or a special value; special values are 'auto', 'unix', 'mac', 'windows', 'unicode'; defaults to 'auto' (discovered in source or 'unix' if no source is specified).

Todo:

*   `eof`          Add a linebreak on the last line, default to false, expect a charactere or use '\n' if value is set to "true"
 */

Stringifier = function(options) {
  var _base, _base1, _base2, _base3, _base4, _base5, _base6, _base7;
  if (options == null) {
    options = {};
  }
  stream.Transform.call(this, options);
  this.options = options;
  if ((_base = this.options).delimiter == null) {
    _base.delimiter = ',';
  }
  if ((_base1 = this.options).quote == null) {
    _base1.quote = '"';
  }
  if ((_base2 = this.options).quoted == null) {
    _base2.quoted = false;
  }
  if ((_base3 = this.options).escape == null) {
    _base3.escape = '"';
  }
  if ((_base4 = this.options).columns == null) {
    _base4.columns = null;
  }
  if ((_base5 = this.options).header == null) {
    _base5.header = false;
  }
  if ((_base6 = this.options).lineBreaks == null) {
    _base6.lineBreaks = null;
  }
  if ((_base7 = this.options).rowDelimiter == null) {
    _base7.rowDelimiter = '\n';
  }
  if (this.countWriten == null) {
    this.countWriten = 0;
  }
  switch (this.options.rowDelimiter) {
    case 'auto':
      this.options.rowDelimiter = null;
      break;
    case 'unix':
      this.options.rowDelimiter = "\n";
      break;
    case 'mac':
      this.options.rowDelimiter = "\r";
      break;
    case 'windows':
      this.options.rowDelimiter = "\r\n";
      break;
    case 'unicode':
      this.options.rowDelimiter = "\u2028";
  }
  return this;
};

util.inherits(Stringifier, stream.Transform);

Stringifier.prototype.headers = function() {
  var k, label, labels;
  labels = this.options.columns;
  if (typeof labels === 'object') {
    labels = (function() {
      var _results;
      _results = [];
      for (k in labels) {
        label = labels[k];
        _results.push(label);
      }
      return _results;
    })();
  }
  labels = this.stringify(labels);
  return stream.Transform.prototype.write.call(this, labels);
};

Stringifier.prototype.end = function(chunk, encoding, callback) {
  if (this.countWriten === 0) {
    this.headers();
  }
  return stream.Transform.prototype.end.apply(this, arguments);
};

Stringifier.prototype.write = function(chunk, encoding, callback) {
  var e, preserve;
  if (chunk == null) {
    return;
  }
  preserve = typeof chunk !== 'object';
  if (!preserve) {
    try {
      this.emit('record', chunk, this.countWriten);
    } catch (_error) {
      e = _error;
      return this.emit('error', e);
    }
    chunk = this.stringify(chunk);
    if (this.options.header || this.countWriten) {
      chunk = this.options.rowDelimiter + chunk;
    }
  }
  if (typeof chunk === 'number') {
    chunk = "" + chunk;
  }
  if (this.options.header && this.countWriten === 0) {
    this.headers();
  }
  if (!preserve) {
    this.countWriten++;
  }
  return stream.Transform.prototype.write.call(this, chunk);
};

Stringifier.prototype._transform = function(chunk, encoding, callback) {
  this.push(chunk);
  return callback();
};


/*

`Stringifier(line)`
-------------------

Convert a line to a string. Line may be an object, an array or a string.
 */

Stringifier.prototype.stringify = function(line) {
  var column, columns, containsLinebreak, containsQuote, containsdelimiter, delimiter, escape, field, i, newLine, quote, regexp, _i, _j, _line, _ref, _ref1;
  if (typeof line !== 'object') {
    return line;
  }
  columns = this.options.columns;
  if (typeof columns === 'object' && columns !== null && !Array.isArray(columns)) {
    columns = Object.keys(columns);
  }
  delimiter = this.options.delimiter;
  quote = this.options.quote;
  escape = this.options.escape;
  if (!Array.isArray(line)) {
    _line = [];
    if (columns) {
      for (i = _i = 0, _ref = columns.length; 0 <= _ref ? _i < _ref : _i > _ref; i = 0 <= _ref ? ++_i : --_i) {
        column = columns[i];
        _line[i] = typeof line[column] === 'undefined' || line[column] === null ? '' : line[column];
      }
    } else {
      for (column in line) {
        _line.push(line[column]);
      }
    }
    line = _line;
    _line = null;
  } else if (columns) {
    line.splice(columns.length);
  }
  if (Array.isArray(line)) {
    newLine = '';
    for (i = _j = 0, _ref1 = line.length; 0 <= _ref1 ? _j < _ref1 : _j > _ref1; i = 0 <= _ref1 ? ++_j : --_j) {
      field = line[i];
      if (typeof field === 'string') {

      } else if (typeof field === 'number') {
        field = '' + field;
      } else if (typeof field === 'boolean') {
        field = field ? '1' : '';
      } else if (field instanceof Date) {
        field = '' + field.getTime();
      }
      if (field) {
        containsdelimiter = field.indexOf(delimiter) >= 0;
        containsQuote = field.indexOf(quote) >= 0;
        containsLinebreak = field.indexOf('\r') >= 0 || field.indexOf('\n') >= 0;
        if (containsQuote) {
          regexp = new RegExp(quote, 'g');
          field = field.replace(regexp, escape + quote);
        }
        if (containsQuote || containsdelimiter || containsLinebreak || this.options.quoted) {
          field = quote + field + quote;
        }
        newLine += field;
      }
      if (i !== line.length - 1) {
        newLine += delimiter;
      }
    }
    line = newLine;
  }
  return line;
};


/*
`stringify([options])`
`stringify(data, [options], callback)`
 */

module.exports = function() {
  var callback, chunks, d, data, options, stringifier, _i, _len;
  if (arguments.length === 3) {
    data = arguments[0];
    options = arguments[1];
    callback = arguments[2];
  } else if (arguments.length === 2) {
    data = arguments[0];
    callback = arguments[1];
  } else if (arguments.length === 1) {
    options = arguments[0];
  }
  if (options == null) {
    options = {};
  }
  stringifier = new Stringifier(options);
  if (data && callback) {
    chunks = [];
    for (_i = 0, _len = data.length; _i < _len; _i++) {
      d = data[_i];
      stringifier.write(d);
    }
    stringifier.on('readable', function() {
      var chunk, _results;
      _results = [];
      while (chunk = stringifier.read()) {
        _results.push(chunks.push(chunk));
      }
      return _results;
    });
    stringifier.on('error', function(err) {
      return callback(err);
    });
    stringifier.on('finish', function() {
      return callback(null, chunks.join(''));
    });
    stringifier.end();
  }
  return stringifier;
};

module.exports.Stringifier = Stringifier;
